Public Class PianoCottura
    'Variabili private, interne alla classe
    Private WithEvents tmr As Timers.Timer
    Private datFabbricazione As DateTime
    Private intFornelli As Integer
    Private blnMetano As Boolean
    Private blnAcceso As Boolean
    Private intPotenza As Integer

    'Costruiamo (fabbrichiamo) il piano cottura specificando il numero di fornelli
    Public Sub New(ByVal Fornelli As Integer)
        'Valori iniziali delle varie caratteristiche
        intFornelli = Fornelli  'Numero di fornelli specificato
        blnMetano = True        'Connesso alla rete del metano
        intPotenza = 50         'Potenza su 50
        blnAcceso = False       'Spento
        datFabbricazione = Now  'Fabbricato ora [quando viene chiamato New()]
        'Avviamo il timer che dopo 5 secondi simuler lo spegnimento anomalo 
        'del fornello [vedi tmr_Elapsed()]
        tmr = New Timers.Timer(5000)
        tmr.Start()
        Console.WriteLine("Piano cottura costruito!")
        Console.WriteLine("Data di fabbricazione: " & Me.DataFabbricazione)
    End Sub

    'Propriet che restituisce il numero di fornelli
    'Non pu essere modificata
    Public ReadOnly Property NumeroFornelli() As Integer
        Get
            Return intFornelli
        End Get
    End Property

    'Propriet che restituisce la data di fabbricazione
    'Non pu essere modificata
    Public ReadOnly Property DataFabbricazione() As Date
        Get
            Return datFabbricazione
        End Get
    End Property

    'Propriet che restituisce se si  connessi alla rete del metano o meno
    'Permette di connettere e disconnettere il piano cottura [si veda Accendi()]
    Public Property ConnessoReteMetano() As Boolean
        Get
            Return blnMetano
        End Get
        Set(ByVal value As Boolean)
            blnMetano = value
            If blnMetano Then
                Console.WriteLine("Connesso alla rete del metano!")
            Else
                Console.WriteLine("Disconnesso alla rete del metano!")
            End If
        End Set
    End Property

    'Propriet che restituisce se il piano cottura ha qualche fornello acceso o meno
    'Non pu essere modificata [Accendi() utilizza infatti 
    'la variabile interna blnAcceso direttamente]
    Public ReadOnly Property Acceso() As Boolean
        Get
            Return blnAcceso
        End Get
    End Property

    'Metodo che imposta la variabile interna [non modificabile al di fuori dell'oggetto] blnAcceso
    'su True ["accende" il fornello], controllando per prima di essere connessi alla rete del metano
    'e in caso negativo, effettua il collegamento impostando la propriet ConnessoReteMetano su True
    Public Sub Accendi()
        If Not Me.ConnessoReteMetano Then
            Me.ConnessoReteMetano = True
        End If
        blnAcceso = True
        Console.WriteLine("Acceso!")
    End Sub

    'Metodo che spegne il fornello semplicemente impostando la variabile interna blnAccesso su False.
    Public Sub Spegni()
        blnAcceso = False
        Console.WriteLine("Spento!")
    End Sub

    'Metodo che imposta la potenza della fiamma su quella specificata e restituisce quella precedente
    Public Function ImpostaPotenzaFiamma(ByVal NuovaPotenza As Integer) As Integer
        Dim intVecchiaPotenza As Integer = intPotenza
        intPotenza = NuovaPotenza
        Console.WriteLine("Nuova potenza: " & intPotenza)
        Return intVecchiaPotenza
    End Function

    'Evento che verr richiamato in caso di spegnimento anomalo della fiamma
    Public Event SpegnimentoAnomaloFuoco()

    'Metodo privato che dopo 5 secondi dalla costruzione richiama l'evneto SpegnimentoAnomaloFuoco() se il fuoco  acceso
    Private Sub tmr_Elapsed(ByVal sender As Object, ByVal e As System.Timers.ElapsedEventArgs) Handles tmr.Elapsed
        If Me.Acceso Then
            RaiseEvent SpegnimentoAnomaloFuoco()
        End If
    End Sub
End Class
